module HaskellUnit where
import System.IO
import Control.Exception
import Data.Maybe
import Data.List

replace [] _ _ = error "Extra.replace, first argument cannot be empty"
replace from to xs | Just xs <- stripPrefix from xs = to ++ replace from to xs
replace from to (x:xs) = x : replace from to xs
replace from to [] = []



runTests tests = do
  putStrLn "<?xml version=\"1.0\" ?>"
  putStrLn "<testsuite>"
  xs <- sequence tests
  putStrLn ("  <results errors=\""++show (length (filter not xs))++"\" tests=\""++show (length xs)++"\" failures=\"0\"/>")
  putStrLn "</testsuite>"


compareResult expected actual = do
  catch 
    (if (expected == actual)
      then return Nothing
      else return (Just ("expected:&lt;"++show expected++"&gt; but was:&lt;"++show actual++"&gt;")))
    (\e->return (Just (replace "\n" " "("Exception in evaluation: "++ (show (e :: SomeException)))))) 


testcase name msg expected actual  = do
    res <- compareResult expected actual
    maybe
      (do
          putStrLn ("  <testcase name=\""++name++"\">")
          hFlush stdout
          putStrLn "  </testcase>"
          hFlush stdout
          return True)
      (\failMessage -> do
          putStrLn ("  <testcase name=\""++name++"\">")
          hFlush stdout
          --let failMessage = "expected:&lt;"++show expected++"&gt; but was:&lt;"++show actual++"&gt;"
          putStrLn ("    <failure message=\"msg: \" type=\"AssertionFailedError\">")
          putStrLn (msg++": "++failMessage)
          putStrLn "    </failure>"
          hFlush stdout
          putStrLn "  </testcase>"
          hFlush stdout
          return False)
      res 